@@ -21,6 +21,7 @@
 #include <dbus/dbus.h>
 #include <glib-object.h>
 #include <string.h>
+#include <unistd.h>
 
 #include "oobs-object.h"
 #include "oobs-object-private.h"
@@ -35,9 +36,8 @@
  * @short_description: Object that represents the current user
  **/
 
-#define SELF_CONFIG_REMOTE_OBJECT "SelfConfig"
+#define SELF_CONFIG_REMOTE_OBJECT "SelfConfig2"
 #define OOBS_SELF_CONFIG_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), OOBS_TYPE_SELF_CONFIG, OobsSelfConfigPrivate))
-#define POLICY_KIT_SELF_ACTION "org.freedesktop.systemtoolsbackends.self.set"
 
 typedef struct _OobsSelfConfigPrivate OobsSelfConfigPrivate;
 
@@ -52,10 +52,9 @@ static void oobs_self_config_init        (OobsSelfConfig      *config);
 static void oobs_self_config_constructed (GObject             *object);
 static void oobs_self_config_finalize    (GObject             *object);
 
-static void oobs_self_config_update     (OobsObject   *object);
-static void oobs_self_config_commit     (OobsObject   *object);
-
-static const gchar * oobs_self_config_get_authentication_action (OobsObject *object);
+static void oobs_self_config_update             (OobsObject   *object);
+static void oobs_self_config_commit             (OobsObject   *object);
+static void oobs_self_config_get_update_message (OobsObject   *object);
 
 
 G_DEFINE_TYPE (OobsSelfConfig, oobs_self_config, OOBS_TYPE_OBJECT);
@@ -72,7 +71,7 @@ oobs_self_config_class_init (OobsSelfConfigClass *class)
 
   oobs_object_class->commit  = oobs_self_config_commit;
   oobs_object_class->update  = oobs_self_config_update;
-  oobs_object_class->get_authentication_action = oobs_self_config_get_authentication_action;
+  oobs_object_class->get_update_message = oobs_self_config_get_update_message;
 
   g_type_class_add_private (object_class,
 			    sizeof (OobsSelfConfigPrivate));
@@ -81,7 +80,13 @@ oobs_self_config_class_init (OobsSelfConfigClass *class)
 static void
 oobs_self_config_init (OobsSelfConfig *config)
 {
-  config->_priv = OOBS_SELF_CONFIG_GET_PRIVATE (config);
+  OobsSelfConfigPrivate *priv;
+
+  priv = OOBS_SELF_CONFIG_GET_PRIVATE (config);
+
+  priv->uid = getuid ();
+
+  config->_priv = priv;
 }
 
 static void
@@ -141,16 +146,9 @@ oobs_self_config_finalize (GObject *object)
 static void
 oobs_self_config_update (OobsObject *object)
 {
-  OobsSelfConfigPrivate *priv;
   OobsObject *users_config;
-  DBusMessage *message;
-  DBusMessageIter iter;
 
-  priv = OOBS_SELF_CONFIG (object)->_priv;
-  message = _oobs_object_get_dbus_message (object);
-
-  dbus_message_iter_init (message, &iter);
-  priv->uid = utils_get_uint (&iter);
+  /* We don't actually need to update, since we joint point to an OobsUser */
 
   users_config = oobs_users_config_get ();
 
@@ -168,7 +166,6 @@ oobs_self_config_commit (OobsObject *object)
   OobsSelfConfigPrivate *priv;
   DBusMessageIter iter, array_iter;
   DBusMessage *message;
-  gchar *passwd;
 
   priv = OOBS_SELF_CONFIG (object)->_priv;
 
@@ -178,11 +175,7 @@ oobs_self_config_commit (OobsObject *object)
   message = _oobs_object_get_dbus_message (object);
   dbus_message_iter_init_append (message, &iter);
 
-  utils_append_uint (&iter, oobs_user_get_uid (priv->user));
-
-  g_object_get (priv->user, "crypted-password", &passwd, NULL);
-  utils_append_string (&iter, passwd);
-  g_free (passwd);
+  utils_append_uint (&iter, priv->uid);
 
   /* GECOS fields */
   dbus_message_iter_open_container (&iter,
@@ -197,12 +190,28 @@ oobs_self_config_commit (OobsObject *object)
   utils_append_string (&array_iter, oobs_user_get_other_data (priv->user));
 
   dbus_message_iter_close_container (&iter, &array_iter);
+
+  utils_append_string (&iter, oobs_user_get_locale (priv->user));
+  /* TODO: use location when the backends support it */
+  utils_append_string (&iter, "");
 }
 
-static const gchar *
-oobs_self_config_get_authentication_action (OobsObject *object)
+/*
+ * We need a custom update message containing the user's UID.
+ */
+static void
+oobs_self_config_get_update_message (OobsObject *object)
 {
-  return POLICY_KIT_SELF_ACTION;
+  OobsSelfConfigPrivate *priv;
+  DBusMessageIter iter;
+  DBusMessage *message;
+
+  priv = OOBS_SELF_CONFIG (object)->_priv;
+
+  message = _oobs_object_get_dbus_message (object);
+  dbus_message_iter_init_append (message, &iter);
+
+  utils_append_uint (&iter, priv->uid);
 }
 
 /**
@@ -236,8 +245,35 @@ oobs_self_config_get_user (OobsSelfConfig *config)
 
   g_return_val_if_fail (OOBS_IS_SELF_CONFIG (config), NULL);
 
-  oobs_object_ensure_update (oobs_users_config_get ());
+  oobs_object_ensure_update (OOBS_OBJECT (config));
   priv = config->_priv;
 
   return priv->user;
 }
+
+
+/**
+ * oobs_self_config_is_user_self:
+ * @config: An #OobsSelfConfig.
+ * @user: An #OobsUser.
+ *
+ * Check whether @user is the user represented by #OobsSelfConfig.
+ *
+ * Return Value: %TRUE if @user is self, %FALSE otherwise.
+ **/
+gboolean
+oobs_self_config_is_user_self (OobsSelfConfig *config,
+                               OobsUser       *user)
+{
+  OobsSelfConfigPrivate *priv;
+
+  g_return_val_if_fail (OOBS_IS_SELF_CONFIG (config), FALSE);
+  g_return_val_if_fail (OOBS_IS_USER (user), FALSE);
+
+  oobs_object_ensure_update (OOBS_OBJECT (config));
+  priv = config->_priv;
+
+  return (priv->user == user);
+}
+
+
