@@ -178,6 +178,7 @@ oobs_service_get_name (OobsService *service)
  * @runlevel: A #OobsServicesRunlevel.
  * @status: status for the service in the given runlevel
  * @priority: priority for the service in the given runlevel.
+ * Use 0 to either keep the previous priority, or get the default value in case the script is new.
  *
  * Sets the configuration of a service for a given runlevel.
  **/
@@ -196,22 +197,21 @@ oobs_service_set_runlevel_configuration (OobsService          *service,
 
   priv = service->_priv;
 
-  if (status == OOBS_SERVICE_IGNORE)
-    g_hash_table_remove (priv->runlevels_config, runlevel);
-  else
+  service_runlevel = g_hash_table_lookup (priv->runlevels_config, runlevel);
+
+  if (!service_runlevel)
     {
-      service_runlevel = g_hash_table_lookup (priv->runlevels_config, runlevel);
-
-      if (!service_runlevel)
-	{
-	  service_runlevel = g_new0 (OobsServiceRunlevel, 1);
-	  g_hash_table_insert (priv->runlevels_config,
-			       runlevel, service_runlevel);
-	}
-
-      service_runlevel->status = status;
-      service_runlevel->priority = priority;
+      service_runlevel = g_new0 (OobsServiceRunlevel, 1);
+      g_hash_table_insert (priv->runlevels_config,
+                           runlevel, service_runlevel);
     }
+
+  service_runlevel->status = status;
+
+  /* Keep previous priority. If the script was not used previously,
+   * the backends will use a default value. */
+  if (priority != 0)
+    service_runlevel->priority = priority;
 }
 
 /**
@@ -240,8 +240,8 @@ oobs_service_get_runlevel_configuration (OobsService          *service,
   service_runlevel = g_hash_table_lookup (priv->runlevels_config, runlevel);
 
   if (status)
-    *status = (service_runlevel) ? service_runlevel->status : OOBS_SERVICE_IGNORE;
+    *status = (service_runlevel) ? service_runlevel->status : OOBS_SERVICE_STOP;
 
   if (priority)
-    *priority = (service_runlevel) ? service_runlevel->priority : -1;
+    *priority = (service_runlevel) ? service_runlevel->priority : 0;
 }
