@@ -25,7 +25,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <crypt.h>
-#include <utmp.h>
+#include <utmpx.h>
 
 #include "oobs-object-private.h"
 #include "oobs-usersconfig.h"
@@ -87,8 +87,9 @@ static void oobs_user_get_property (GObject      *object,
 				    GValue       *value,
 				    GParamSpec   *pspec);
 
-static void oobs_user_commit    (OobsObject *object);
-static void oobs_user_update    (OobsObject *object);
+static void oobs_user_commit             (OobsObject *object);
+static void oobs_user_update             (OobsObject *object);
+static void oobs_user_get_update_message (OobsObject *object);
 
 enum
 {
@@ -125,6 +126,7 @@ oobs_user_class_init (OobsUserClass *class)
 
   oobs_class->commit = oobs_user_commit;
   oobs_class->update = oobs_user_update;
+  oobs_class->get_update_message = oobs_user_get_update_message;
 
   /* override the singleton check */
   oobs_class->singleton = FALSE;
@@ -620,6 +622,24 @@ oobs_user_commit (OobsObject *object)
   dbus_message_iter_close_container (&iter, &struct_iter);
 }
 
+/*
+ * We need a custom update message containing the user's UID.
+ */
+static void
+oobs_user_get_update_message (OobsObject *object)
+{
+  OobsUserPrivate *priv;
+  DBusMessageIter iter;
+  DBusMessage *message;
+
+  priv = OOBS_USER (object)->_priv;
+
+  message = _oobs_object_get_dbus_message (object);
+  dbus_message_iter_init_append (message, &iter);
+
+  utils_append_string (&iter, priv->username);
+}
+
 static void
 oobs_user_update (OobsObject *object)
 {
@@ -1232,7 +1252,7 @@ oobs_user_set_locale (OobsUser *user, const gchar *locale)
 gboolean
 oobs_user_get_active (OobsUser *user)
 {
-  struct utmp *entry;
+  struct utmpx *entry;
   const gchar *login;
   gboolean match = FALSE;
 
@@ -1240,14 +1260,16 @@ oobs_user_get_active (OobsUser *user)
 
   login = oobs_user_get_login_name (user);
 
-  while (!match && (entry = getutent ()) != NULL)
-    {
-      match = (entry->ut_type == USER_PROCESS &&
-	       strcmp (entry->ut_user, login) == 0);
-    }
+  setutxent ();
+
+  while (!match && (entry = getutxent ()) != NULL)
+     {
+       match = (entry->ut_type == USER_PROCESS &&
+               strcmp (entry->ut_user, login) == 0);
+     }
 
   /* close utmp */
-  endutent ();
+  endutxent ();
 
   return match;
 }
